package requests

import "time"

// ListRequest represents the request for retrieving a list of folders.
type ListFolderRequest struct {
	// Folder name.
	// Required: false
	Name string `url:"name,omitempty" json:"name,omitempty"`

	// Folder icon type.
	// Required: false
	IconType string `url:"icon_type,omitempty" json:"icon_type,omitempty"`

	// Folder ID of the parent folder. "root" if there is no parent folder.
	// Required: false
	ParentID string `url:"parent_id,omitempty" json:"parent_id,omitempty"`

	// Template for filtering by any text field.
	// Required: false
	FilterText string `url:"filter_text,omitempty" json:"filter_text,omitempty"`

	// List of columns that will be used by FilterText.
	// Required: false
	FilterColumns string `url:"filter_columns,omitempty" json:"filter_columns,omitempty"`

	// Filter entities created before this date.
	// Required: false
	CreatedBefore time.Time `url:"created_before,omitempty" json:"created_before,omitempty"`

	// Filter entities created after this date.
	// Required: false
	CreatedAfter time.Time `url:"created_after,omitempty" json:"created_after,omitempty"`

	// Field name or list of field names used for sorting. Ascending sort is default. For descending sort, use "-" before the name.
	// Required: false
	Sort []string `url:"sort,omitempty" json:"sort,omitempty"`

	// Visibility options: "visible" - Only active folders, "deleted" - Only removed folders, "all" - All folders.
	// Required: false
	Visibility string `url:"visibility,omitempty" json:"visibility,omitempty" validate:"omitempty,visibility"`

	// Number of the returning page.
	// Required: false
	Page int `url:"page,omitempty" json:"page,omitempty"`

	// Number of items on the page.
	// Required: false
	Limit int `url:"limit,omitempty" json:"limit,omitempty"`
}
